((Drupal, drupalSettings, once) => {
  /**
   * Collection of functions to respond to postMessage events.
   */
  const iFrameActions = {
    /**
     * Ajax click handler for Layout Paragraphs UI elements in iframe.
     * @param {Object} settings The ajax settings.
     */
    drupalAjax(settings) {
      Drupal.ajax(settings).execute();
    },
    /**
     * Closes the modal for a component.
     * @param {Object} settings The settings for the modal.
     */
    closeModal(settings) {
      const componentType = settings.componentType.replace(/_/g, '-');
      const dialogElement = document.querySelector(
        `mercury-dialog:has([data-drupal-selector="edit-layout-paragraphs-component-form-${componentType}"])`,
      );
      if (!dialogElement) {
        return;
      }
      dialogElement.close();
    },
    /**
     * Syncs changes to an element.
     * @param {Object} data The data to sync.
     */
    syncChanges(data) {},
    /**
     * Runs a set of ajax commands.
     * @param {Object} data The ajax commands.
     */
    ajaxCommands(data) {
      const { commands, status } = data;
      const ajaxObj = Drupal.ajax({ url: '' });
      const ajaxCommands = new Drupal.AjaxCommands();
      Object.keys(commands || {}).reduce((executionQueue, key) => {
        return executionQueue
          .then(() => {
            const { command } = commands[key];
            if (command && ajaxCommands[command]) {
              return ajaxCommands[command](ajaxObj, commands[key], status);
            }
          })
          .catch(console.error);
      }, Promise.resolve());
    },
    ajaxPreviewPageState(pageState) {
      drupalSettings.ajaxPreviewPageState = pageState;
    },
    previewUpdated(data) {
      const eventObject = new CustomEvent(`mercuryEditorPreviewUpdated`, {
        bubbles: true,
        detail: data,
      });
      document.dispatchEvent(eventObject);
    },
    updateStatesUi(data) {
      if (data.stateIndex === 0) {
        document.querySelector('#me-undo').classList.add('disabled');
      } else {
        document.querySelector('#me-undo').classList.remove('disabled');
      }
      if (data.stateCount === 0 || data.stateIndex === data.stateCount - 1) {
        document.querySelector('#me-redo').classList.add('disabled');
      } else {
        document.querySelector('#me-redo').classList.remove('disabled');
      }
    },
    componentSelected(data) {
      const { uuid } = data;
      const component = document.querySelector(`[data-uuid="${uuid}"]`);
      if (component) {
        component.scrollIntoView({ behavior: 'smooth', block: 'center' });
        const event = new CustomEvent('lpb-component:focus', {
          bubbles: true,
          cancelable: true,
          detail: { uuid },
        });
        component.dispatchEvent(event);
      }
    },
  };
  Drupal.behaviors.mercuryEditorPostMessagesListener = {
    attach() {
      // Listen for window.postMessage() to handle iFrame behavior.
      if (once('me-msg-listener', 'html').length) {
        window.addEventListener('message', (e) => {
          if (iFrameActions[e.data.type]) {
            iFrameActions[e.data.type](e.data.settings);
          }
        });
      }
    },
  };
})(Drupal, drupalSettings, once);
