<?php

namespace Drupal\mercury_editor\Element;

use Drupal\Core\Render\RendererInterface;
use Drupal\Core\Render\Element\RenderElementBase;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Layout\LayoutPluginManagerInterface;
use Drupal\layout_paragraphs\LayoutParagraphsLayout;
use Drupal\Core\Entity\EntityTypeBundleInfoInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\layout_paragraphs\LayoutParagraphsComponent;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\layout_paragraphs\LayoutParagraphsLayoutTempstoreRepository;
use Drupal\Tests\layout_paragraphs\Functional\LayoutParagraphsTest;

/**
 * Element that renders the Mercury Editor component outline.
 *
 * @RenderElement("mercury_editor_component_outline")
 */
class MercuryEditorComponentOutline extends RenderElementBase implements ContainerFactoryPluginInterface {

  /**
   * {@inheritdoc}
   */
  public function __construct(
    array $configuration,
    $plugin_id,
    $plugin_definition,
    protected LayoutParagraphsLayoutTempstoreRepository $layoutParagraphsTempstore,
    protected EntityTypeManagerInterface $entityTypeManager,
    protected LayoutPluginManagerInterface $layoutPluginManager,
    protected RendererInterface $renderer,
    protected EntityTypeBundleInfoInterface $entityTypeBundleInfo,
  ) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('layout_paragraphs.tempstore_repository'),
      $container->get('entity_type.manager'),
      $container->get('plugin.manager.core.layout'),
      $container->get('renderer'),
      $container->get('entity_type.bundle.info')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getInfo() {
    return [
      '#entity' => NULL,
      '#pre_render' => [
        [$this, 'preRender'],
      ],
      '#attributes' => [
        'class' => [
          'me-component-outline',
        ],
      ],
    ];
  }

  /**
   * Pre-render callback: renders the outline.
   *
   * @param array $element
   *   The render array for the element.
   *
   * @return array
   *   The render array for the element.
   */
  public function preRender(array $element) {
    $entity = $element['#entity'];
    $element['#type'] = 'container';
    $element['#attributes']['class'][] = 'me-component-outline';
    $element['components'] = [];
    if ($entity) {
      foreach ($entity->lp_storage_keys as $key) {
        $layout = $this->layoutParagraphsTempstore->getWithStorageKey($key);
        $element['components'][] = [
          '#type' => 'html_tag',
          '#tag' => 'ul',
          '#attributes' => [
            'class' => [
              'me-component-outline__components',
            ],
          ],
          'components' => $this->renderComponentList($layout, $layout->getRootComponents()),
        ];
      }
    }
    $element['#attached']['library'][] = 'mercury_editor/component_outline';
    return $element;
  }

  /**
   * Renders the layout outline.
   *
   * @param \Drupal\layout_paragraphs\LayoutParagraphsLayout $layout
   *   The layout to render.
   * @param \Drupal\layout_paragraphs\LayoutParagraphsComponent[] $components
   *   The layout to render.
   */
  protected function renderComponentList(
    LayoutParagraphsLayout $layout,
    array $components,
  ) {
    $list = [];
    foreach ($components as $component) {
      $list_item = [
        '#type' => 'html_tag',
        '#tag' => 'li',
        '#attributes' => [
          'class' => [
            'me-component-outline__component',
          ],
        ],
        'label' => $this->componentLabel($component, $layout),
      ];
      if ($component->isLayout()) {
        $list_item['regions'] = $this->renderLayoutRegions($component, $layout);
      }
      $list[] = $list_item;
    }
    return $list;
  }

  /**
   * Renders the layout regions.
   *
   * @param \Drupal\layout_paragraphs\LayoutParagraphsComponent $component
   *   The component to render.
   * @param \Drupal\layout_paragraphs\LayoutParagraphsLayout $layout
   *   The layout to render.
   */
  protected function renderLayoutRegions(LayoutParagraphsComponent $component, LayoutParagraphsLayout $layout) {
    $section = $layout->getLayoutSection($component->getEntity());
    $layout_plugin_id = $section->getLayoutId();
    $definition = $this->layoutPluginManager->getDefinition($layout_plugin_id);
    $regions = $definition->getRegions();
    $render_array = [
      '#type' => 'html_tag',
      '#tag' => 'ul',
      '#attributes' => [
        'class' => [
          'me-component-outline__regions',
        ],
      ],
      'regions' => [],
    ];
    foreach ($regions as $id => $region) {
      $region_render_array =  [
        '#type' => 'html_tag',
        '#tag' => 'li',
        '#attributes' => [
          'class' => [
            'me-component-outline__region',
          ],
        ],
        'label' => [
          '#type' => 'html_tag',
          '#tag' => 'span',
          '#attributes' => [
            'class' => [
              'me-component-outline__region-label',
            ],
          ],
          '#value' => $region['label'] . ' Region',
        ],
      ];
      $components = $section->getComponentsForRegion($id);
      if ($components) {
        $region_render_array['components'] = [
          '#type' => 'html_tag',
          '#tag' => 'ul',
          '#attributes' => [
            'class' => [
              'me-component-outline__components',
            ],
          ],
          'components' => $this->renderComponentList($layout, $components),
        ];
      }
      $render_array['regions'][] = $region_render_array;
    }
    return $render_array;
  }

  /**
   * Returns the component label.
   *
   * @param \Drupal\layout_paragraphs\LayoutParagraphsComponent $component
   *   The component to get the label for.
   * @param \Drupal\layout_paragraphs\LayoutParagraphsLayout $layout
   *   The layout to get the label for.
   */
  protected function componentLabel(LayoutParagraphsComponent $component, LayoutParagraphsLayout $layout) {
    $entity = $component->getEntity();
    $label = $this->getBundleLabel($entity);
    if ($component->isLayout()) {
      $section = $layout->getLayoutSection($entity);
      $layout_plugin_id = $section->getLayoutId();
      $definition = $this->layoutPluginManager->getDefinition($layout_plugin_id);
      $label .= ' - ' . $definition->getLabel();
    }
    return [
      '#type' => 'html_tag',
      '#tag' => 'button',
      '#value' => $label,
      '#attributes' => [
        'type' => 'button',
        'data-uuid' => $component->getEntity()->uuid(),
        'class' => [
          'me-component-outline__component-label',
        ],
      ],
    ];
  }

  /**
   * Get the bundle label for the entity.
   *
   * @param \Drupal\Core\Entity\ContentEntityInterface $entity
   *   The entity to get the bundle label for.
   *
   * @return string|null
   *   The bundle label, or NULL if the entity does not have a bundle.
   */
  protected function getBundleLabel($entity) {
    $entity_type_id = $entity->getEntityTypeId();
    $bundle = $entity->bundle();
    $bundle_entity_type = \Drupal::entityTypeManager()
      ->getDefinition($entity_type_id)
      ->getBundleEntityType();

    if ($bundle_entity_type) {
      $bundle_entity = \Drupal::entityTypeManager()
        ->getStorage($bundle_entity_type)
        ->load($bundle);
      return $bundle_entity ? $bundle_entity->label() : NULL;
    }

    return NULL;
  }

}
