<?php

namespace Drupal\mercury_editor;

use Drupal\Core\TempStore\PrivateTempStoreFactory;
use Drupal\layout_paragraphs\LayoutParagraphsLayout;
use Drupal\layout_paragraphs\LayoutParagraphsLayoutTempstoreRepository;

/**
 * Decorates the LayoutParagraphsLayoutTempstoreRepository class.
 */
class LayoutParagraphsLayoutTempstoreRepositoryDecorator extends LayoutParagraphsLayoutTempstoreRepository {

  /**
   * Constructs a new LayoutParagraphsLayoutTempstoreRepositoryDecorator object.
   *
   * @param \Drupal\Core\TempStore\PrivateTempStoreFactory $tempstore_private
   *   The private tempstore factory service.
   * @param \Drupal\layout_paragraphs\LayoutParagraphsLayoutTempstoreRepository $originalService
   *   The original layout paragraphs tempstore repository service.
   * @param \Drupal\mercury_editor\MercuryEditorTempstore $mercuryEditorTempstore
   *   The mercury editor tempstore service.
   */
  public function __construct(
    PrivateTempStoreFactory $tempstore_private,
    protected LayoutParagraphsLayoutTempstoreRepository $originalService,
    protected MercuryEditorTempstore $mercuryEditorTempstore) {
    $this->tempStoreFactory = $tempstore_private;
  }

  /**
   * Extend set function to also set changes in Mercury Editor tempstore.
   *
   * This ensures that every time a change is made to a Layout Paragraphs Layout
   * instance, the change is applied to the parent entity and set in the Mercury
   * Editor Tempstore as well, preventing the ME entity from becoming out of
   * sync with the Layout Paragraphs instance.
   *
   * @param \Drupal\layout_paragraphs\LayoutParagraphsLayout $layout_paragraphs_layout
   *   The layout paragraphs layout.
   */
  public function set(LayoutParagraphsLayout $layout_paragraphs_layout) {
    $is_mercury_editor_context = $layout_paragraphs_layout->getSetting('mercury_editor_context');
    if ($is_mercury_editor_context) {
      $parent_entity = $layout_paragraphs_layout->getEntity();
      $field_name = $layout_paragraphs_layout->getFieldName();
      $parent_uuid = $parent_entity->uuid();
      $me_entity = $this->mercuryEditorTempstore->get($parent_uuid);
      if ($me_entity && $me_entity->$field_name) {
        $me_entity->$field_name = $layout_paragraphs_layout->getParagraphsReferenceField();
        // @todo Is this necessary?
        // $layout_paragraphs_layout->setParagraphsReferenceField($me_entity->$field_name);
        $this->mercuryEditorTempstore->set($me_entity);
      }
    }
    return $this->originalService->set($layout_paragraphs_layout);
  }

  /**
   * Get a layout paragraphs layout from the tempstore.
   *
   * @param \Drupal\layout_paragraphs\LayoutParagraphsLayout $layout_paragraphs_layout
   *   A layout paragraphs layout instance.
   *
   * @return \Drupal\layout_paragraphs\LayoutParagraphsLayout
   *   The layout paragraphs layout instance from the tempstore.
   */
  public function get(LayoutParagraphsLayout $layout_paragraphs_layout) {
    $layout_paragraphs_layout = $this->originalService->get($layout_paragraphs_layout);
    $is_mercury_editor_context = $layout_paragraphs_layout->getSetting('mercury_editor_context');
    if ($is_mercury_editor_context) {
      $field_name = $layout_paragraphs_layout->getFieldName();
      $me_entity = $this->getMercuryEditorEntity($layout_paragraphs_layout);
      if ($me_entity && $me_entity->$field_name) {
        $layout_paragraphs_layout->setParagraphsReferenceField($me_entity->$field_name);
        $this->originalService->set($layout_paragraphs_layout);
      }
    }
    return $layout_paragraphs_layout;
  }

  /**
   * Get the Mercury Editor entity from the tempstore.
   *
   * @param \Drupal\layout_paragraphs\LayoutParagraphsLayout $layout_paragraphs_layout
   *   The layout paragraphs layout.
   *
   * @return object
   *   The Mercury Editor entity.
   */
  protected function getMercuryEditorEntity(LayoutParagraphsLayout $layout_paragraphs_layout) {
    $parent_entity = $layout_paragraphs_layout->getEntity();
    $parent_uuid = $parent_entity->uuid();
    return $this->mercuryEditorTempstore->get($parent_uuid);
  }

}
